@@ -78,6 +78,8 @@ void main(int argc, char *argv[])
     int sel;
     struct timeval timeout;
     fd_set readfds;
+    long tstamp = ticks();
+    long ts;
 
     argc_save = argc;
     argv_save = argv;
@@ -120,8 +122,6 @@ void main(int argc, char *argv[])
 	syslog(LOG_INFO,"Diald initial setup completed.");
 
     /* main loop */
-    timeout.tv_sec = PAUSETIME;
-    timeout.tv_usec = 0;
     while (!terminate) {
 	/* wait up to a second for an event */
         FD_ZERO(&readfds);
@@ -129,6 +129,11 @@ void main(int argc, char *argv[])
 	    FD_SET(fifo_fd,&readfds);
         FD_SET(proxy_mfd,&readfds);
         FD_SET(snoopfd,&readfds);
+	/* Compute the likely timeout for the next second boundary */
+	ts = tstamp + PAUSETIME*CLK_TCK - ticks();
+	if (ts < 0) ts = 0;
+    	timeout.tv_sec = ts/CLK_TCK;
+    	timeout.tv_usec = 1000*(ts%CLK_TCK)/CLK_TCK;
 	sel = select(100,&readfds,0,0,&timeout);
 	if (sel > 0) {
 	    /* read user commands off the fifo */
@@ -140,11 +145,18 @@ void main(int argc, char *argv[])
 	    /* deal with packets coming into the pty proxy link */
 	    if (FD_ISSET(proxy_mfd,&readfds)) proxy_read();
 	}
-	if (timeout.tv_sec == 0 && timeout.tv_usec == 0) {
+	/* check if ticks() has advanced a second since last check.
+	 * This is immune to wall clock skew because we use the ticks count.
+	 */
+	ts = tstamp + PAUSETIME*CLK_TCK - ticks();
+	if (ts <= 0) {
+	    tstamp = ticks();
 	    fire_timers();
-	    /* advance the clock 1 second */
-	    timeout.tv_sec = PAUSETIME;
-	    timeout.tv_usec = 0;
+	    /* Advance a second on state machine timeouts.
+	     * Under high load this can get stretched out.
+	     * Even under low load it is likely to have 1% error.
+	     * This doesn't bother me enough to bother changing things.
+	     */
 	    if (state_timeout > 0) state_timeout--;
 	    if (debug&DEBUG_TICK)
 	        syslog(LOG_DEBUG,"--- tick --- state %d block %d state_timeout %d",state,blocked,state_timeout);
