@@ -34,13 +34,16 @@ void idle_filter_init()
 
     if (mode == MODE_SLIP) {
 	sprintf(snoop_dev,"sl%d",link_iface);
-    } else {
-	sprintf(snoop_dev,"ppp%d",link_iface);
-    }
+     } else if (mode == MODE_PPP) {
+       sprintf(snoop_dev,"ppp%d",link_iface);
+     } else if (mode == MODE_DEV) {
+       sprintf(snoop_dev,"%s",device);
+      }
     if (debug) syslog(LOG_INFO,"Changed snoop device to %s",snoop_dev);
     txtotal = rxtotal = 0;
 
     /* try to bind the snooping socket to a particular device */
+    memset(&to,0,sizeof(to));
     to.sa_family = AF_INET;
     strcpy(to.sa_data,snoop_dev);
     /* This bind may fail if the kernel isn't recent enough. */
@@ -68,6 +71,7 @@ void idle_filter_proxy()
     struct sockaddr to;
 
     if (fwdfd != -1) {
+        if (debug) syslog(LOG_INFO,"Closed fwdfd");
 	close(fwdfd);
 	fwdfd = -1;
     }
@@ -76,6 +80,7 @@ void idle_filter_proxy()
 
     /* try to bind the snooping socket to a particular device */
     /* Most likely this should close the old socket and open a new one first */
+    memset(&to,0,sizeof(to));
     to.sa_family = AF_INET;
     strcpy(to.sa_data,snoop_dev);
     /* This bind may fail if the kernel isn't recent enough. */
@@ -91,7 +96,7 @@ void idle_filter_proxy()
 void filter_read()
 {
     struct sockaddr from;
-    int from_len = sizeof(struct sockaddr);
+    size_t from_len = sizeof(struct sockaddr);
     int len;
 
     if ((len = recvfrom(snoopfd,packet,4096,0,&from,&from_len)) > 0) {
@@ -157,6 +162,21 @@ int queue_empty()
     return ctl_firewall(IP_FW_QCHECK,&req);
 }
 
+int fw_wait()
+{
+    struct firewall_req req;
+    req.unit = fwunit;
+    return ctl_firewall(IP_FW_WAIT,&req);
+}
+
+int fw_reset_wait()
+{
+    struct firewall_req req;
+    req.unit = fwunit;
+    return ctl_firewall(IP_FW_RESET_WAITING,&req);
+}
+
+
 void print_filter_queue(int sig)
 {
     struct firewall_req req;
@@ -175,5 +195,5 @@ void monitor_queue()
     ctl_firewall(IP_FW_MCONN,&req);
     sprintf(buf,"LOAD\n%d\n%d\n",itxtotal,irxtotal);
     itxtotal = irxtotal = 0;
-    mon_write(buf,strlen(buf));
+    mon_write(MONITOR_LOAD,buf,strlen(buf));
 }
