@@ -7,6 +7,11 @@
  */
 
 #include "diald.h"
+#ifdef __GLIBC__
+typedef u_int8_t __u8;
+typedef u_int16_t __u16;
+typedef u_int32_t __u32;
+#endif
 
 static FW_unit units[FW_NRUNIT];
 static int initialized = 0;
