@@ -171,16 +171,13 @@ void free_tokens(void)
 
 void init_prule(FW_ProtocolRule *rule)
 {
-    rule->protocol = 0;
+    memset(rule,0,sizeof(*rule));
 }
 
 void init_filter(FW_Filter *filter)
 {
+    memset(filter,0,sizeof(*filter));
     filter->times = cslot;
-    filter->prule = 0;
-    filter->log = 0;
-    filter->count = 0;
-    filter->timeout = 0;
 }
 
 void eat_whitespace(void)
@@ -374,18 +371,19 @@ void parse_new_prule_name(void)
 
 void parse_protocol_name(FW_ProtocolRule *prule)
 {
-    struct protoent *proto;
+    int proto;
     if (token->type == TOK_STR) {
 	if (strcmp(token->str,"any") == 0)
 	    { prule->protocol = 255; ADVANCE; return; }
-        if ((proto = getprotobyname(token->str)))
-	    { prule->protocol = proto->p_proto; ADVANCE; return; }
+        if ((proto = getprotocol(token->str)))
+	    { prule->protocol = proto; ADVANCE; return; }
 	parse_error("Expecting a protocol name or 'any'.");
-    } else if (token->type != TOK_NUM) {
+    } else if (token->type == TOK_NUM) {
 	int p;
 	sscanf(token->str,"%i",&p);
 	if (p > 254) parse_error("Expecting number from 0-254.");
 	prule->protocol = p;
+	ADVANCE;
     } else
         parse_error("Expecting a string or a number.");
 }
@@ -452,18 +450,18 @@ int parse_rvalue(void)
 	    parse_error("Bad inet address specification.");
 	ADVANCE; return v;
     } else if (token->type == TOK_STR) {
-	struct protoent *proto;
-	struct servent *serv;
-	if ((proto = getprotobyname(token->str))) {
-	    ADVANCE; return proto->p_proto;
+	int proto;
+	int serv;
+	if ((proto = getprotocol(token->str))) {
+	    ADVANCE; return proto;
 	} else if (strncmp("udp.",token->str,4) == 0) {
-	    if ((serv = getservbyname(token->str+4,"udp"))) {
-	 	ADVANCE; return htons(serv->s_port);
+	    if ((serv = getservice(token->str+4,"udp"))) {
+	 	ADVANCE; return serv;
 	    }
 	    parse_error("Not a known udp service port.");
 	} else if (strncmp("tcp.",token->str,4) == 0) {
-	    if ((serv = getservbyname(token->str+4,"tcp"))) {
-	 	ADVANCE; return htons(serv->s_port);
+	    if ((serv = getservice(token->str+4,"tcp"))) {
+	 	ADVANCE; return serv;
 	    }
 	    parse_error("Not a known tcp service port.");
 	}
