@@ -30,6 +30,12 @@ unsigned long timestamp()
    return times(&buf)/CLK_TCK;
 }
 
+unsigned long ticks()
+{
+   struct tms buf;
+   return times(&buf);
+}
+
 void init_timer(struct timer_lst * timer)
 {
     timer->next = NULL;
